//////////////////////////////////////////////////////////////////////
// scene.cpp: implementation of the scene class.
//
//////////////////////////////////////////////////////////////////////

#include "scene.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

scene::scene()
{
	int j;
	extInit();
	font0 = NULL;

	ReadInit("data/init.txt");

	//////////////////////////////////////
	// smerniky na instancie na triedy nastavim na NULL, aby som ich potom nemazal
	lietadlo = NULL;
	teapot = NULL;
	c = NULL;

	//zistime podporu cube mapping
	if(!ext.ARB_texture_cube_map)
	{	MessageBox(hWnd,english ? "Cube mapping unsupported !":"Cube mapping nie je podporovany !", "ERROR", MB_OK);error=1;return;}

	font0 = new Font("data/font.bmp",&error);		// vytvorenie fontu
	if(error)return;

	//////////////////////////////////////
	// prepinace

	fps = 200;							// v pripade ze nieco zavisy od fps
	clip_distance = vzdialena_orezavacia_rovina;
	svetlo=0; texturovanie=0; ciary=0; kurzor=1;
	
	//////////////////////////////////////
	// nacitanie textur
	j=0;
	
	cube_map.setCubeForCopyToTexture( renderableTexture, renderableTexture, 0, filterForCubeMap);

	light.load("data/svetlo.jpg",0,2);

	LoadTextureSkyBox( "data/skybox/","jpg", skybox, 0);
	//////////////////////////////////////
	// vytvarame instancie pouzivanych tried
	c = new camera;
	lietadlo = new C3ds("data/lietadlo/pitts.3ds");
	teapot = new C3ds("data/teapot.3ds");

	if(error)return;
	InitGL();		// nastavenie z-buffra,svetla,...
	lietad=0;cajnik=0;anuloid=0;gula=0;
	BuildDisplayLists();
}

scene::~scene()
{
//	int i;
	//////////////////////////////////////
	// mazeme instancie tried
	if(lietadlo!=NULL)delete lietadlo;
	if(teapot!=NULL)delete teapot;
	if(font0!=NULL)delete font0;
	if(c!=NULL)delete c;
	//////////////////////////////////////
	// mazeme dipley listy
	glDeleteLists(lietad,1);
	glDeleteLists(cajnik,1);
	glDeleteLists(anuloid,1);
	glDeleteLists(gula,1);

	KillGL();
}

void scene::InitGL(void)
{
	if(!kurzor)ShowCursor(0);
	if(!fullscreen)
	{
		RECT		WindowRect;					// miesto pre velkost okna
		WindowRect.left=(long)0;
		WindowRect.top=(long)0;
		GetClientRect(hWnd,&WindowRect);				//zistenie rozmerov uzivatelskej casti okna
		ReSizeGLScene(WindowRect.right,WindowRect.bottom);	//nastavenie perspektivy
	}
	else ReSizeGLScene(screen_x, screen_y);	//definovana v tomto subore

	glEnable(GL_TEXTURE_2D);		// Povolenie zobrazovanie textr
	glShadeModel(GL_SMOOTH);		/*nastavuje tienovanie
	GL_SMOOTH - objekty su tienovane t.j. prechody medzi farbami bodov su plinule (gouradovo (s)tienovanie)
	GL_FLAT   - objekty nie su tienovane ziadne plinule prechody */

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f); //specifikuje hodnoty na cistenie pomocou glClear par.: red, green, blue, alpha (cierny podklad)
//	glClearColor(0.6f, 0.6f, 0.6f, 0.0f); //specifikuje hodnoty na cistenie pomocou glClear par.: red, green, blue, alpha (cierny podklad)
	glClearDepth(1.0f);  //specifikuje hodnoty na cistenie pomocou glClear par.: depth (hlbka)
	
	/* ZAPNUTIE HLBKOVEHO TESTU */
	glEnable(GL_DEPTH_TEST);		// zapne hbkov test
	glDepthFunc(GL_LEQUAL);			// typ hbkovho testu
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// relnejie perspektvne vpoty

	GLfloat LightAmbient[]	=	{ 0.2f, 0.2f, 0.2f, 1.0f };		//svetlo ktore osvetluje zo vsetkych stran rovnako (okolite svetlo)
	GLfloat LightDiffuse[]	=	{ 0.8f, 0.8f, 0.8f, 1.0f };		//smerove, alebo bodove svetlo - svetlo svieti z pozicie zadanej pomocou premennej LightPosition
	GLfloat LightPosition[4]=	{ 0.0f, 0.0f, 1.0f, 0.0f };		//smer difuzneho svetla - ak je 4 zlozka 0 - smerove svetlo, ak nie je 0 potom s jedna o bodove svetlo so suradnicami x,y,z
	
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	glLightfv ( GL_LIGHT0, GL_POSITION, LightPosition  );		// pozicia difuzneho svetla
	glEnable  ( GL_LIGHT0 );									// aktivovanie svetla 0
	
	GLfloat mat_diffuse[]	= { 1.0f, 1.0f, 1.0f, 1.0f };
//	GLfloat mat_specular[]	= { 0.9f, 0.9f, 0.9f, 1.0f };
//	GLfloat mat_shininess[] = { 128.0f };

	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE , mat_diffuse);
//	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
//	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

//	GLfloat	fogColor[4] = {0.6f,0.6f,0.6f,1.0f};		// Fog Color
//	glFogfv(GL_FOG_COLOR, fogColor);					// farba hmly
//	glFogi(GL_FOG_MODE, GL_EXP);						// mod hmly
//	glFogf(GL_FOG_DENSITY, 0.1f);						// hustota hmly
//	glFogi(GL_FOG_MODE,GL_LINEAR);
//	glHint(GL_FOG_HINT, GL_DONT_CARE);					// dalsie nastavenie
//	glFogf(GL_FOG_START, 0.5*clip_distance);			// startovacia vzdialenost
//	glFogf(GL_FOG_END, clip_distance);					// ukoncovacia vzdialenost
//	glEnable(GL_FOG);

	glCullFace( GL_BACK );
//	glEnable(GL_CULL_FACE );
	glColor3f(1.0f,1.0f,1.0f);				// biela farba	
}

void scene::KillGL()
{
	if(!kurzor)ShowCursor(1);
}


void scene::Kurzor(void)			// zisti poziciu mysi a zobrazy kurzor
{
/*	GetCursorPos(&MouseBod);		//nacitanie pozicie mysi
	glBegin(GL_LINES);
		glVertex2i(MouseBod.x+velkost_kurzoru+1,screen_y-1-MouseBod.y);
		glVertex2i(MouseBod.x-velkost_kurzoru+1,screen_y-1-MouseBod.y);
		glVertex2i(MouseBod.x+1,screen_y-1-MouseBod.y+velkost_kurzoru);
		glVertex2i(MouseBod.x+1,screen_y-1-MouseBod.y-velkost_kurzoru);
	glEnd();
*/
	glBegin(GL_LINES);
		glVertex2i(mouse_x+velkost_kurzoru+1,mouse_y);
		glVertex2i(mouse_x-velkost_kurzoru+1,mouse_y);
		glVertex2i(mouse_x+1,mouse_y+velkost_kurzoru);
		glVertex2i(mouse_x+1,mouse_y-velkost_kurzoru);
	glEnd();
}

void scene::ReSizeGLScene(int width, int height)
{
	sirka = width;
	vyska = height;
	ReSizeGLScene_3D(width, height);
}

void scene::ReSizeGLScene_3D(int width, int height)		// funkcia pre zmenu velkosti okna
{										// vola sa v pripade zmeny velkosti okna, zmenu z(na) fullscreen
	if (height==0) height=1;
	glViewport(0,0,width,height);		// Viewport (pohlad)
	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
//	gluPerspective(uhol_kamery,(GLfloat)width/(GLfloat)height,blizka_orezavacia_rovina,vzdialena_orezavacia_rovina);
	gluPerspective(uhol_kamery,(GLfloat)width/(GLfloat)height,blizka_orezavacia_rovina,clip_distance);
	/* gluPerspective - nastavuje perspektivu kamery, ako bude vypadat priestor z pohladu kamery.
	1. parameter urcuje zorn uhol kamery. 2.param. - gldAspect je pomer vky a rky zobrazovanej plochy.
	3. par. udava vzdialenost od ktorej je vidiet vykreslovane objekty
	4. par. udava vzdialenost do ktorej je vidiet vykreslovane objekty */
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();					// Reset The Modelview Matrix
}

void scene::ReSizeGLScene_2D(int width, int height)		// funkcia pre zmenu velkosti okna
{
	if (height==0) height=1;
	glViewport(0,0,width,height);		// Viewport (pohlad)
	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	glOrtho(0,screen_x,0,screen_y,-1,1);	// nastavuje standardne pravouhle premietanie
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();					// Reset The Modelview Matrix
}

void scene::Prepni_do_3D(void)			// rychle prepnutie do 3D
{
	glEnable(GL_DEPTH_TEST );

	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	//gluPerspective(uhol_kamery,(float)sirka/(float)vyska,blizka_orezavacia_rovina,vzdialena_orezavacia_rovina);
	gluPerspective(uhol_kamery,(GLfloat)sirka/(GLfloat)vyska,blizka_orezavacia_rovina,clip_distance);
	/* gluPerspective - nastavuje perspektivu kamery, ako bude vypadat priestor z pohladu kamery.
	1. parameter urcuje zorn uhol kamery. 2.param. - gldAspect je pomer vky a rky zobrazovanej plochy.
	3. par. udava vzdialenost od ktorej je vidiet vykreslovane objekty
	4. par. udava vzdialenost do ktorej je vidiet vykreslovane objekty */
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();
}

void scene::Prepni_do_2D(void)			// rychle prepnutie do 2D
{
	if(svetlo)glDisable(GL_LIGHTING);					// vypne svetlo
	if(ciary)glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);	// vypne ciarovy rezim
	if(!texturovanie)glEnable(GL_TEXTURE_2D);			// zapne texturovanie
	glDisable(GL_DEPTH_TEST );

	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	glOrtho(0,screen_x,0,screen_y,-1,1);// nastavuje standardne pravouhle premietanie
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();					// Reset The Modelview Matrix
}

void scene::SaveScreen_RAW()
{
	FILE *sub;
	unsigned char* buf = new unsigned char[3*screen_x*screen_y];
	unsigned char* pom = new unsigned char[3*screen_x*screen_y];
	char file[30];

	if(buf==NULL || pom==NULL)
	{
		if(buf==NULL)delete[] buf;
		if(pom==NULL)delete[] pom;
		return;
	}

	glPixelStorei( GL_PACK_ALIGNMENT, 1);
	glReadPixels(0, 0, screen_x, screen_y, GL_RGB, GL_UNSIGNED_BYTE, buf);
	for(int i=0; i<screen_y; i++)
	{
		for(int j=0; j<3*screen_x; j++)
		{
			pom[3*screen_x*(screen_y-i-1)+j] = buf[i*3*screen_x+j];
		}
	}
	delete[] buf;
	
	for(int c=0;c<99;c++)
	{
		char pom[20];
		strcpy(file,"screen");
		if(c<10)strcat(file,"0");
		strcat(file,itoa(c,pom,10));
		strcat(file,".raw");
		sub = fopen(file,"r");
		if(sub==NULL)c=100;
		if(sub!=NULL)fclose(sub);
	}
	
	sub = fopen(file,"w");
	fwrite(pom, 3*screen_x*screen_y, 1, sub);
	fclose(sub);
	delete[] pom;
}

void scene::SaveScreen_TGA()
{
	FILE *sub;
	unsigned char* buf = new unsigned char[3*screen_x*screen_y];
	char file[30];
	unsigned char ctmp = 0, type, mode, rb;
	short int width = screen_x, height = screen_y, itmp = 0;
	unsigned char pixelDepth = 24;

	if(buf==NULL )return;

	glPixelStorei( GL_PACK_ALIGNMENT, 1);
	glReadPixels(0, 0, screen_x, screen_y, GL_RGB, GL_UNSIGNED_BYTE, buf);

	for(int c=0;c<99;c++)
	{
		char pom[20];
		strcpy(file,"screen");
		if(c<10)strcat(file,"0");
		strcat(file,itoa(c,pom,10));
		strcat(file,".tga");
		sub = fopen(file,"r");
		if(sub==NULL)c=100;
		if(sub!=NULL)fclose(sub);
	}
	
	sub = fopen(file,"wb");
	
	mode = pixelDepth / 8;
	if ((pixelDepth == 24) || (pixelDepth == 32))
		type = 2;
	else
		type = 3;

	// tu sa musi prehodit R->B - pozri specifikaciu TGA formatu.
	if (mode >= 3) {
		for (int i = 0; i < width * height * mode; i += mode)
		{
			rb = buf[i];
			buf[i] = buf[i+2];
			buf[i+2] = rb;
		}
	}

	// tu sa zapisuje hlavicka
	fwrite(&ctmp, sizeof(unsigned char), 1, sub);
	fwrite(&ctmp, sizeof(unsigned char), 1, sub);

	fwrite(&type, sizeof(unsigned char), 1, sub);

	fwrite(&itmp, sizeof(short int), 1, sub);
	fwrite(&itmp, sizeof(short int), 1, sub);
	fwrite(&ctmp, sizeof(unsigned char), 1, sub);
	fwrite(&itmp, sizeof(short int), 1, sub);
	fwrite(&itmp, sizeof(short int), 1, sub);

	fwrite(&width, sizeof(short int), 1, sub);
	fwrite(&height, sizeof(short int), 1, sub);
	fwrite(&pixelDepth, sizeof(unsigned char), 1, sub);

	fwrite(&ctmp, sizeof(unsigned char), 1, sub);
	
	fwrite(buf, 3*screen_x*screen_y, 1, sub);

	delete[] buf;
	fclose(sub);
}

void scene::Paint_axes(float size, float r, float g, float b)
{
	GLfloat LightAmbient[4];		//svetlo ktore osvetluje zo vsetkych stran rovnako (okolite svetlo)
	GLfloat LightDiffuse[4];		//smerove, alebo bodove svetlo - svetlo svieti z pozicie zadanej pomocou premennej LightPosition

	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	glEnable  ( GL_LIGHT0 );
	glDisable(GL_TEXTURE_2D);
	glEnable( GL_LIGHTING);
	GLUquadricObj *quad;
	quad = gluNewQuadric();

	LightAmbient[0]=0.2f*r; LightAmbient[1]=0.2f*g; LightAmbient[2]=0.2f*b; LightAmbient[3]=0;
	LightDiffuse[0]=0.8f*r; LightDiffuse[1]=0.8f*g; LightDiffuse[2]=0.8f*b; LightDiffuse[3]=0;
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	gluSphere( quad, size*0.03f, 10, 10);
	LightAmbient[0]=0.2f; LightAmbient[1]=0.0f; LightAmbient[2]=0.0f;
	LightDiffuse[0]=0.8f; LightDiffuse[1]=0.0f; LightDiffuse[2]=0.0f;
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	
	glPushMatrix();
		glTranslatef(size,0.0f,0.f);
		glRotatef(-90.f,0,1,0);
		gluCylinder(quad, 0, size*0.05f, size*0.2f, 20, 1);
		glRotatef(90.f,0,1,0);
		glTranslatef( -size*0.2f,0.0f,0.f);
		glRotatef(-90.f,0,1,0);
		gluDisk(quad, 0, size*0.05f, 20, 1);
	glPopMatrix();
	LightAmbient[0]=0;
	LightAmbient[1]=0.2f;
	LightDiffuse[0]=0;
	LightDiffuse[1]=0.8f;
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	
	glPushMatrix();
		glRotatef(90,0,0,1);
		glTranslatef(size,0.0f,0.f);
		glRotatef(-90.f,0,1,0);
		gluCylinder(quad, 0, size*0.05f, size*0.2f, 20, 1);
		glRotatef(90.f,0,1,0);
		glTranslatef( -size*0.2f,0.0f,0.f);
		glRotatef(-90.f,0,1,0);
		gluDisk(quad, 0, size*0.05f, 20, 1);
	glPopMatrix();
	
	LightAmbient[1]=0;
	LightAmbient[2]=0.2f;
	LightDiffuse[1]=0;
	LightDiffuse[2]=0.8f;
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	glPushMatrix();
		glRotatef(90,0,-1,0);
		glTranslatef(size,0.0f,0.f);
		glRotatef(-90.f,0,1,0);
		gluCylinder(quad, 0, size*0.05f, size*0.2f, 20, 1);
		glRotatef(90.f,0,1,0);
		glTranslatef( -size*0.2f,0.0f,0.f);
		glRotatef(-90.f,0,1,0);
		gluDisk(quad, 0, size*0.05f, 20, 1);
	glPopMatrix();

	gluDeleteQuadric(quad);
	glDisable( GL_LIGHTING);

	glLineWidth( size*0.2f+1.0f );
	glBegin(GL_LINES);
		glColor3f(1,0,0);
		glVertex3f(-0.1f*size,0,0);
		glVertex3f(0.8f*size,0,0);

		glColor3f(0,1,0);
		glVertex3f(0,-0.1f*size,0);
		glVertex3f(0,0.8f*size,0);

		glColor3f(0,0,1);
		glVertex3f(0,0,-0.1f*size);
		glVertex3f(0,0,0.8f*size);
	glEnd();
	glLineWidth( 1.0f );
	glColor3f(1,1,1);

	LightAmbient[0]=0.2f;
	LightAmbient[1]=0.2f;
	LightAmbient[2]=0.2f;
	LightDiffuse[0]=0.8f;
	LightDiffuse[1]=0.8f;
	LightDiffuse[2]=0.8f;
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla

	if(svetlo)glEnable(GL_LIGHTING);
	if(texturovanie)glEnable(GL_TEXTURE_2D);
}

void scene::Vypis_maticu(const CMatrix4 &m, int stlpec, int riadok, char* nadpis)
{
	glEnable(GL_TEXTURE_2D);
	font0->Begin();
	if(nadpis!=NULL)
	{
		font0->Print(stlpec, riadok++, nadpis,0);
	}
	for(int i=0;i<16;i++)
	{
		gcvt( (double)m.m[i], 2, text);			//prevod realneho cisla na text
		//	a00  a04  a08  a12 
		//  a01  a05  a09  a13
		//  a02  a06  a10  a14
		//  a03  a07  a11  a15
		font0->Print(stlpec+(i/4)*10,riadok+(i%4),text,0);
	}
	font0->End();
}

void scene::Vypis_vektor(const CVector &a, int stlpec, int riadok, char* meno)
{
	glEnable(GL_TEXTURE_2D);
	font0->Begin();
	if(meno!=NULL)
	{
		font0->Print(stlpec, riadok, meno,0);
		stlpec += strlen(meno)+2;
	}
	
	gcvt( (double)a.x, 2, text);
	font0->Print(stlpec,riadok,text,0);
	stlpec+=10;
	gcvt( (double)a.y, 2, text);
	font0->Print(stlpec,riadok,text,0);
	stlpec+=10;
	gcvt( (double)a.z, 2, text);
	font0->Print(stlpec,riadok,text,0);

	font0->End();
}

void scene::SaveScreen_BMP()
{
	BITMAPFILEHEADER fileheader;
	BITMAPINFOHEADER infoheader;

	FILE *sub;
	unsigned char* buf = new unsigned char[3*screen_x*screen_y];
	char file[30];

	if(buf==NULL )return;
	
	glPixelStorei( GL_PACK_ALIGNMENT, 1);
	glReadPixels(0, 0, screen_x, screen_y, GL_RGB, GL_UNSIGNED_BYTE, buf);

	for(int c=0;c<999;c++)
	{
		char pom[20];
		strcpy(file,"screen");
		if(c<100)strcat(file,"0");
		if(c<10)strcat(file,"0");
		strcat(file,itoa(c,pom,10));
		strcat(file,".bmp");
		sub = fopen(file,"r");
		if(sub==NULL)break;
		if(sub!=NULL)fclose(sub);
	}
	
	int line_size = 3*screen_x;
	if(line_size%4)line_size += 4 - line_size%4;
	
	fileheader.bfType = 0x4D42; // Magic identifier   - "BM"	| identifikacia BMP suboru musi byt "BM"
	fileheader.bfSize = 3*screen_x*screen_y+sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);	// File size in bytes			| velkos suboru v byte
	fileheader.bfReserved1 = 0;
	fileheader.bfReserved2 = 0;
	fileheader.bfOffBits = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);	// Offset to image data, bytes	| posun na zaciatok dat
	
	infoheader.biSize = sizeof(BITMAPINFOHEADER);	// Header size in bytes			| velkost hlavicky BITMAPINFOHEADER
	infoheader.biWidth = screen_x;	// Width of image			| sirka obrazka - sizeX
	infoheader.biHeight = screen_y;	// Height of image			| vyska obrazka - sizeY
	infoheader.biPlanes = 1;		// Number of colour planes	| pocet farebnych rovin musi bit 1
	infoheader.biBitCount = 24;		// Bits per pixel			| bitov na pixel moze bit 1,4,8,24
	infoheader.biCompression = 0;	// Compression type			| typ compresie , 0 - bez kompresie
	infoheader.biSizeImage = line_size*infoheader.biHeight ;	// Image size in bytes		| velkost obrazka v byte
	infoheader.biXPelsPerMeter = 0;	// Pixels per meter X		| pixelov na meter v smere x
	infoheader.biYPelsPerMeter = 0;	// Pixels per meter Y		| pixelov na meter v smere y
	infoheader.biClrUsed = 0;		// Number of colours		| pocet  farieb v palete, ak 0 vsetky su pouzivane
	infoheader.biClrImportant = 0;	// Important colours		| dolezite farby v palete, ak 0 vsetky su dolezite
	
	sub = fopen(file,"wb");
	fwrite( &fileheader, sizeof(BITMAPFILEHEADER), 1, sub);
	fwrite( &infoheader, sizeof(BITMAPINFOHEADER), 1, sub);
	
	if(line_size!=3*screen_x)
	{
		unsigned char* p = new unsigned char[line_size*screen_y];
		for(int y=0; y<screen_y; y++)
		{
			for(int x=0; x<screen_x; x++)
			{
				p[y*line_size+x*3+0] = buf[(y*screen_x+x)*3+2];
				p[y*line_size+x*3+1] = buf[(y*screen_x+x)*3+1];
				p[y*line_size+x*3+2] = buf[(y*screen_x+x)*3+0];
			}
		}
		fwrite( p, line_size*screen_y, 1, sub);
		delete[] p;
	}
	else
	{
		// musime zmenit z RGB na BGR pretoze bmp pouziva BGR
		for (int i = 0; i < screen_x*screen_y*3; i += 3)
		{
			int rb = buf[i];
			buf[i] = buf[i+2];
			buf[i+2] = rb;
		}
		fwrite( buf, 3*screen_x*screen_y, 1, sub);
	}

	delete[] buf;
	fclose(sub);
}


void scene::DrawSkyBox(texture *tex, float size)
{
	CVector p,n,s,t;
	// pozerame na +x
	tex[0].glBindTexture2D();
	n.set(-1,0,0);
	s.set(0,0,2*size);
	t.set(0,2*size,0);
	p = CVector(size,0,0) -0.5*s -0.5*t;
	DrawQuad( p, n, s, t);
	// pozerame na -x
	tex[1].glBindTexture2D();
	n.set(1,0,0);
	s.set(0,0,-2*size);
	p = CVector(-size,0,0) -0.5*s -0.5*t;
	DrawQuad( p, n, s, t);
	// pozerame na +y
	tex[2].glBindTexture2D();
	n.set(0,-1,0);
	s.set(2*size,0,0);
	t.set(0,0,2*size);
	p = CVector(0,size,0) -0.5*s -0.5*t;
	DrawQuad( p, n, s, t);
	// pozerame na -y
	tex[3].glBindTexture2D();
	n.set(0,1,0);
	s.set(2*size,0,0);
	t.set(0,0,-2*size);
	p = CVector(0,-size,0) -0.5*s -0.5*t;
	DrawQuad( p, n, s, t);
	// pozerame na +z
	tex[4].glBindTexture2D();
	n.set(0,0,1);
	s.set(-2*size,0,0);
	t.set(0,2*size,0);
	p = CVector(0,0,size) -0.5*s -0.5*t;
	DrawQuad( p, n, s, t);
	// pozerame na -z
	tex[5].glBindTexture2D();
	n.set(0,0,1);
	s.set(2*size,0,0);
	p = CVector(0,0,-size) -0.5*s -0.5*t;
	DrawQuad( p, n, s, t);
}

void scene::DrawQuad(CVector p, CVector n, CVector s, CVector t)
{
	glBegin(GL_TRIANGLE_STRIP);				// 3  4  poradie bodov
		glNormal3fv(n.v);					// 1  2
		glTexCoord2f(0,0);glVertex3f( p.v[0], p.v[1], p.v[2]);
		glTexCoord2f(1,0);glVertex3f( p.v[0]+s.v[0],p.v[1]+s.v[1],p.v[2]+s.v[2]);
		glTexCoord2f(0,1);glVertex3f( p.v[0]+t.v[0],p.v[1]+t.v[1],p.v[2]+t.v[2]);
		glTexCoord2f(1,1);glVertex3f( p.v[0]+s.v[0]+t.v[0],p.v[1]+s.v[1]+t.v[1],p.v[2]+s.v[2]+t.v[2]);
	glEnd();
}

int scene::LoadTextureSkyBox(char *directory, char *extension, texture *tex, int repeat, int filter, int compression)
{
	// right, left, top, bottom, back, front
	char* filename[6];
	int OK=1;

	for(int j=0; j<6; j++)
	{
		filename[j] = new char[strlen(directory)+strlen(extension)+7];
		strcpy( filename[j], directory);
	}

	strcat( filename[0], "right.");
	strcat( filename[1], "left.");
	strcat( filename[2], "top.");
	strcat( filename[3], "bottom.");
	strcat( filename[4], "back.");
	strcat( filename[5], "front.");
	for( j=0; j<6; j++) strcat( filename[j], extension);

	strlwr(extension);				// Convert a string to lowercase
	
	for( j=0; j<6; j++)
	{
		if(j==3)
		{
			FILE* f;
			f = fopen(filename[3],"rb");
			if(f==NULL)
			{
				strcpy( filename[3], directory);
				strcat( filename[3], "down.");
				strcat( filename[3], extension);
			}
			else fclose(f);
		}
		if(!tex[j].load(filename[j], repeat, filter, compression))OK=0;
	}
	return OK;
}

int scene::ReadInit(char *filename)
{
	FILE* sub=NULL;
	char temp[100];
	char varname[100],varvalue[100];
	int ret=0;

	sub = fopen( filename, "rb");
	if(sub == NULL)return 0;

	renderableTexture=0;
	filterForCubeMap=1;
	while(GetVariable( temp, sub, varname, varvalue, 100))
	{
		float fl=(float)atof(varvalue);
		int in=atoi(varvalue);
	//	strlwr(varname);				// Convert a string to lowercase
		if(!strcmp("renderableTexture",varname))renderableTexture=in;
		if(!strcmp("filterForCubeMap",varname))filterForCubeMap=in;
	}
	fclose(sub);
	if(renderableTexture==0)renderableTexture=128;
	if(filterForCubeMap>3||filterForCubeMap<0)filterForCubeMap=1;
	return 1;
}

int scene::GetVariable(char *temp, FILE *sub, char *varname, char *varvalue, int size_strings)
{
	if(fgets(temp, size_strings, sub) == NULL)
	{
		temp[0] = NULL;
		return 0;
	}
	varname[0]=NULL;
	varvalue[0]=NULL;
	char rovnasa=0;
	for(int i=0,j=0,k=0; temp[i]!=NULL&&temp[i]!='\n'&&temp[i]!='/'; i++)		// prejdeme vsetky znaky
	{
		if(temp[i]==' ' || temp[i]=='\t')continue;
		if(temp[i]=='='){ rovnasa=1;continue;}
		if(rovnasa)
		{
			varvalue[k]=temp[i]; k++;
		}
		else
		{
			varname[j]=temp[i]; j++;
		}
	}
	varname[j]=NULL;
	varvalue[k]=NULL;
	return 1;
}


