// texture.cpp: implementation of the texture class.
//
//////////////////////////////////////////////////////////////////////

#include "texture.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

texture::texture()
{
	id = 0;
	name = NULL;
	extens = NULL;
}

texture::~texture()
{
	glDeleteTextures( 1, &id);
	if(name!=NULL)delete[]name;
	if(extens!=NULL)delete[]extens;
	if(im.data!=NULL) delete []im.data;
}

/* 
 char *filename, int repeat, int filter, int compression
 repeat : 0-clamp, 1-repeat
 filter :
 0 - nearest
 1 - linear
 2 - bilinear, linear_mipmap_nearest
 3 - trilinear, linear_mipmap_linear
*/
int texture::load(char *filename, int repeat, int filter, int compression)
{
	char p[5];

	char* pdest = strrchr( filename, '.' );		// find '.'
	if( pdest[0]!=NULL)pdest++;
	strncpy( p, pdest, 4);
	strlwr(p);				// Convert a string to lowercase
	
	if(im.data!=NULL) delete[]im.data;
	im.data = NULL;

	if( !strcmp(p,"bmp"))		im = LoadBMP( filename);
	else if( !strcmp(p,"jpg"))	im = LoadJPG( filename);
	else if( !strcmp(p,"jpeg"))	im = LoadJPG( filename);
	else if( !strcmp(p,"tga"))	im = LoadTGA( filename);
	
	if(im.data==NULL)
	{
		MessageBox( hWnd, filename, "Nenjden sbor", MB_OK );
		error = 1;
		return 0;
	}

	// uprava obrazku aby bol 2^ x 2^
	if( !isPow2(im.sizeX) || !isPow2(im.sizeY) )
	{
		int newx=1,newy=1;
		for( int newx2=im.sizeX; newx2 ;newx2>>=1)newx<<=1;		// newx = 2^n, newx>im.sizeX
		for( int newy2=im.sizeY; newy2 ;newy2>>=1)newy<<=1;		// newy = 2^m, newy>im.sizeY
		if( isPow2(im.sizeX) ) newx = im.sizeX;
		if( isPow2(im.sizeY) ) newy = im.sizeY;

		IMAGE out=resize( im, newx, newy);
		if( im.data!=NULL)delete [] im.data;
		if( out.data==NULL)return 0;
		im = out;
	}

	// uprava obrazku aby velkost nebola vecsia ako maximalna velkost
	while(im.sizeX>ext.max.max_texture_size && im.sizeY>ext.max.max_texture_size)
	{
		IMAGE ret = resizeToHalfXY( im);
		delete [] im.data;		
		if(ret.data==NULL)return 0;
		im = ret;
	}
	while(im.sizeX>ext.max.max_texture_size )
	{
		IMAGE ret = resizeToHalfX( im);
		delete [] im.data;		
		if(ret.data==NULL)return 0;
		im = ret;
	}
	while( im.sizeY>ext.max.max_texture_size)
	{
		IMAGE ret = resizeToHalfY( im);
		delete [] im.data;		
		if(ret.data==NULL)return 0;
		im = ret;
	}

	if(name!=NULL)delete[]name;
	name = new char[strlen(filename)+1];
	strcpy(name,filename);

	if(id!=0)glDeleteTextures( 1, &id);
	glGenTextures(1, &id);
	glBindTexture(GL_TEXTURE_2D, id);

	if(repeat)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	}
	else
	{
		if(ext.EXT_texture_edge_clamp)
		{	// vyuzijeme rozsirenie GL_SGI_texture_edge_clamp ( GL_EXT_texture_edge_clamp )
			// pri texturavani v blizkosti hrany sa farba hrany nepouziva, texturuje sa iba z textury
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere v (zvislom) neopakuju
		}
		else
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );		// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );		// nastavuje ze textury sa v smere v (zvislom) neopakuju
		}
	}

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	switch(filter)
	{
	case 0:		// nearest
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		break;
	case 1:		// linear
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		break;
	case 2:		// bilinear
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		break;
	case 3:		// trilinear
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		break;
	default:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}

	unsigned int format;
	int internalformat;
	if(im.planes==1)
	{
		format = GL_ALPHA;
		if(compression && ext.ARB_texture_compression) internalformat = GL_COMPRESSED_ALPHA_ARB;
		else internalformat = GL_ALPHA8;
	}
	else if(im.planes==3)
	{
		if(im.isBGR&&ext.EXT_bgra) format = GL_BGR_EXT;
		else 
		{
			if(im.isBGR)BGRtoRGB(im);
			format = GL_RGB;
		}
		if(compression && ext.ARB_texture_compression) internalformat = GL_COMPRESSED_RGB_ARB;
		else internalformat = GL_RGB8;
	}
	else if(im.planes==4)
	{
		if(im.isBGR&&ext.EXT_bgra) format = GL_BGRA_EXT;
		else 
		{
			if(im.isBGR)BGRtoRGB(im);
			format = GL_RGBA;
		}
		if(compression && ext.ARB_texture_compression) internalformat = GL_COMPRESSED_RGBA_ARB;
		else internalformat = GL_RGBA8;
	}

	if(filter<2)
	{
		glTexImage2D(GL_TEXTURE_2D, 0, internalformat, im.sizeX, im.sizeY, 0, format, GL_UNSIGNED_BYTE, im.data);
	}
	else
	{			// mipmap
		if(ext.SGIS_generate_mipmap)
		{
			glHint(GL_GENERATE_MIPMAP_HINT_SGIS, GL_NICEST);
			glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, TRUE);
			glTexImage2D(GL_TEXTURE_2D, 0, internalformat, im.sizeX, im.sizeY, 0, format, GL_UNSIGNED_BYTE, im.data);
		}
		else
		gluBuild2DMipmaps(GL_TEXTURE_2D, internalformat, im.sizeX, im.sizeY, format, GL_UNSIGNED_BYTE, im.data);
	}
	
	if(im.data) delete [] im.data;
	im.data=NULL;
	return 1;
}

IMAGE texture::LoadBMP(char *filename)
{
	FILE* f;
	IMAGE i;
	BITMAPFILEHEADER fileheader;
	BITMAPINFOHEADER infoheader;
	unsigned char* data;

	f = fopen(filename,"rb");
	if(f==NULL)return i;

	if(!fread( &fileheader, sizeof(BITMAPFILEHEADER), 1, f))
	{	fclose(f); return i; }

	if(!fread( &infoheader, sizeof(BITMAPINFOHEADER), 1, f))
	{	fclose(f); return i; }

	// kontrola ci je to BMP subor, 'BM'  == 0x4D42
	if(fileheader.bfType!=0x4D42){	fclose(f); return i; }

	// nacitavame iba obrazky bez kompresie
	if(infoheader.biCompression){	fclose(f); return i; }
	
	switch(infoheader.biBitCount)
	{
	case 1: return LoadBMP1bit( f, i, fileheader, infoheader);
	case 4: return LoadBMP4bit( f, i, fileheader, infoheader);
	case 8: return LoadBMP8bit( f, i, fileheader, infoheader);
	case 24: break;
	case 32: break;
	default:
		fclose(f); return i;
	}

	int line_size = infoheader.biWidth*infoheader.biBitCount/8;
	if(line_size%4)line_size += 4 - line_size%4;
	int data_size = line_size*infoheader.biHeight;
	//	data_size = fileheader.bfSize-sizeof(BITMAPFILEHEADER)-infoheader.biSize;
	data = new unsigned char[data_size];
	if(data==NULL){	fclose(f); return i; }
	if(!fread( data, data_size, 1, f))
	{	
		fclose(f);
		delete [] data;
		return i; 
	}
	fclose(f);
	
	i.sizeX=infoheader.biWidth;
	i.sizeY=infoheader.biHeight;
	i.planes=infoheader.biBitCount/8;
	i.isBGR=1;

	if( (i.sizeX*i.planes)%4==0 ) 
	{
		i.data = data;
		return i;
	}
	
	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL){	fclose(f); delete [] data; return i; }
	
	line_size = i.sizeX*i.planes;
	if(line_size%4)line_size += 4 - line_size%4;	

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(i.sizeX*y+x)*i.planes+2]=data[line_size*y+x*i.planes+0];
			i.data[(i.sizeX*y+x)*i.planes+1]=data[line_size*y+x*i.planes+1];
			i.data[(i.sizeX*y+x)*i.planes+0]=data[line_size*y+x*i.planes+2];
			if(i.planes==4)i.data[(i.sizeX*y+x)*i.planes+3]=data[line_size*y+x*i.planes+3];
		}
	}
	i.isBGR = 0;
	delete [] data;

	return i;
}

// These defines are used to tell us about the type of TARGA file it is
#define TGA_RGB		 2		// This tells us it's a normal RGB (really BGR) file
#define TGA_A		 3		// This tells us it's a ALPHA file
#define TGA_RLE		10		// This tells us that the targa is Run-Length Encoded (RLE)

IMAGE texture::LoadTGA(char *filename)
{
	IMAGE i;

	// code from www.gametutorials.com
	WORD width = 0, height = 0;			// The dimensions of the image
	byte length = 0;					// The length in bytes to the pixels
	byte imageType = 0;					// The image type (RLE, RGB, Alpha...)
	byte bits = 0;						// The bits per pixel for the image (16, 24, 32)
	FILE *pFile = NULL;					// The file pointer
	int channels = 0;					// The channels of the image (3 = RGA : 4 = RGBA)
	int stride = 0;						// The stride (channels * width)
		
	// Open a file pointer to the targa file and check if it was found and opened 
	if((pFile = fopen(filename, "rb")) == NULL) return i;
		
	fread(&length, sizeof(byte), 1, pFile);
	fseek(pFile,1,SEEK_CUR); 
	fread(&imageType, sizeof(byte), 1, pFile);
	fseek(pFile, 9, SEEK_CUR); 

	// Read the width, height and bits per pixel (16, 24 or 32)
	fread(&width,  sizeof(WORD), 1, pFile);
	fread(&height, sizeof(WORD), 1, pFile);
	fread(&bits,   sizeof(byte), 1, pFile);
	
	// Now we move the file pointer to the pixel data
	fseek(pFile, length + 1, SEEK_CUR); 

	// Check if the image is RLE compressed or not
	if(imageType != TGA_RLE)
	{
		// Check if the image is a 24 or 32-bit image
		if(bits == 24 || bits == 32)
		{
			// Calculate the channels (3 or 4) - (use bits >> 3 for more speed).
			// Next, we calculate the stride and allocate enough memory for the pixels.
			channels = bits / 8;
			stride = channels * width;
			i.data = new unsigned char[stride * height];

			// Load in all the pixel data line by line
			for(int y = 0; y < height; y++)
			{
				// Store a pointer to the current line of pixels
				unsigned char *pLine = &(i.data[stride * y]);

				// Read in the current line of pixels
				fread(pLine, stride, 1, pFile);
			}
			i.isBGR=1;
		}
		// Check if the image is a 16 bit image (RGB stored in 1 unsigned short)
		else if(bits == 16)
		{
			unsigned short pixels = 0;
			int r=0, g=0, b=0;

			// Since we convert 16-bit images to 24 bit, we hardcode the channels to 3.
			// We then calculate the stride and allocate memory for the pixels.
			channels = 3;
			stride = channels * width;
			i.data = new unsigned char[stride * height];

			// Load in all the pixel data pixel by pixel
			for(int j = 0; j < width*height; j++)
			{
				// Read in the current pixel
				fread(&pixels, sizeof(unsigned short), 1, pFile);
				
				// To convert a 16-bit pixel into an R, G, B, we need to
				// do some masking and such to isolate each color value.
				// 0x1f = 11111 in binary, so since 5 bits are reserved in
				// each unsigned short for the R, G and B, we bit shift and mask
				// to find each value.  We then bit shift up by 3 to get the full color.
				b = (pixels & 0x1f) << 3;
				g = ((pixels >> 5) & 0x1f) << 3;
				r = ((pixels >> 10) & 0x1f) << 3;
				
				// This essentially assigns the color to our array and swaps the
				// B and R values at the same time.
				i.data[j * 3 + 0] = r;
				i.data[j * 3 + 1] = g;
				i.data[j * 3 + 2] = b;
			}
		}	
		else
			return i;
	}
	// Else, it must be Run-Length Encoded (RLE)
	else
	{
		int j=0;
		byte rleID = 0;
		int colorsRead = 0;
		channels = bits / 8;
		stride = channels * width;

		i.data = new unsigned char[stride * height];
		byte *pColors = new byte [channels];

		while(j < width*height)
		{
			fread(&rleID, sizeof(byte), 1, pFile);
			
			if(rleID < 128)
			{
				rleID++;
				while(rleID)
				{
					fread(pColors, sizeof(byte) * channels, 1, pFile);
					i.data[colorsRead + 0] = pColors[2];
					i.data[colorsRead + 1] = pColors[1];
					i.data[colorsRead + 2] = pColors[0];

					if(bits == 32)i.data[colorsRead + 3] = pColors[3];
					j++;
					rleID--;
					colorsRead += channels;
				}
			}
			else
			{
				rleID -= 127;
				fread(pColors, sizeof(byte) * channels, 1, pFile);

				while(rleID)
				{
					i.data[colorsRead + 0] = pColors[2];
					i.data[colorsRead + 1] = pColors[1];
					i.data[colorsRead + 2] = pColors[0];

					if(bits == 32)i.data[colorsRead + 3] = pColors[3];
					j++;
					rleID--;
					colorsRead += channels;
				}
				
			}
				
		}
	}
	fclose(pFile);

	i.planes = channels;
	i.sizeX    = width;
	i.sizeY    = height;
	Flip( i);

	return i;
}

IMAGE texture::LoadJPG(char *filename)
{
	IMAGE i;
	i = ::LoadJPG( filename);
	Flip( i);
	return i;
}

void texture::BGRtoRGB( IMAGE &i)
{
	if(!i.isBGR || i.planes<3)return;
	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			unsigned char pom = i.data[(i.sizeX*y+x)*i.planes+2];
			i.data[(i.sizeX*y+x)*i.planes+2]=i.data[(i.sizeX*y+x)*i.planes+0];
			i.data[(i.sizeX*y+x)*i.planes+0]=pom;
		}
	}
	i.isBGR=0;
}

void texture::Flip( IMAGE &i)		// hore - dole
{
	if(i.data==NULL)return;
	unsigned char *data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(data==NULL)return;

	for(int y=0; y<i.sizeY; y++)
	{
		memcpy( &data[i.planes*i.sizeX*y], &i.data[i.planes*i.sizeX*(i.sizeY-y-1)], i.planes*i.sizeX );
	}
	delete [] i.data;
	i.data = data;
}

int texture::loadCube(char *directory, char *extension, int filter, int compression)
{
	// right, left, top, bottom, back, front
	char* filename[6];
	IMAGE i[6];

	if( !ext.ARB_texture_cube_map) return 0;
	for(int j=0; j<6; j++)
	{
		filename[j] = new char[strlen(directory)+strlen(extension)+7];
		strcpy( filename[j], directory);
	}

	strcat( filename[0], "right.");
	strcat( filename[1], "left.");
	strcat( filename[2], "top.");
	strcat( filename[3], "bottom.");
	strcat( filename[4], "back.");
	strcat( filename[5], "front.");
	for( j=0; j<6; j++) strcat( filename[j], extension);

	if(extens!=NULL)delete[]extens;
	extens = new char[strlen(extension)+1];
	strcpy(extens,extension);

	strlwr(extension);				// Convert a string to lowercase
	
	for( j=0; j<6; j++)
	{
		if( !strcmp(extension,"bmp"))		i[j] = LoadBMP( filename[j]);
		else if( !strcmp(extension,"jpg"))	i[j] = LoadJPG( filename[j]);
		else if( !strcmp(extension,"jpeg"))	i[j] = LoadJPG( filename[j]);
		else if( !strcmp(extension,"tga"))	i[j] = LoadTGA( filename[j]);
	}

	if(i[3].data==NULL)			// ak sa nieje bottom, mozno bude down
	{
		strcpy( filename[3], directory);
		strcat( filename[3], "down.");
		strcat( filename[3], extension);
		if( !strcmp(extension,"bmp"))		i[3] = LoadBMP( filename[3]);
		else if( !strcmp(extension,"jpg"))	i[3] = LoadJPG( filename[3]);
		else if( !strcmp(extension,"jpeg"))	i[3] = LoadJPG( filename[3]);
		else if( !strcmp(extension,"tga"))	i[3] = LoadTGA( filename[3]);
	}
	int OK=1;
	for( j=0; j<6; j++)
	{
		if(i[j].data==NULL)
		{
			OK=0;
			MessageBox( hWnd, filename[j], "Nenjden sbor", MB_OK );
		}
	}
	
	if(!OK)
	{
		for( j=0; j<6; j++)if(i[j].data!=NULL)delete[] i[j].data;
		error=1;
		return 0;
	}

	for( j=0; j<6; j++)
	{
		// uprava obrazku aby bol 2^x x 2^x
		if( !isPow2(i[j].sizeX) || !isPow2(i[j].sizeY || i[j].sizeX!=i[j].sizeY) )
		{
			int newx=1,newy=1;
			for( int newx2=i[j].sizeX; newx2 ;newx2>>=1)newx<<=1;
			for( int newy2=i[j].sizeY; newy2 ;newy2>>=1)newy<<=1;
			if( isPow2(i[j].sizeX) ) newx = i[j].sizeX;
			if( isPow2(i[j].sizeY) ) newy = i[j].sizeY;
			if(newx!=newy) newx<newy ? newx=newy : newy=newx;

			IMAGE out=resize( i[j], newx, newy);
			if( i[j].data!=NULL)delete [] i[j].data;
			if( out.data==NULL)return 0;
			i[j] = out;
		}

		// uprava obrazku aby velkost nebola vecsia ako maximalna velkost
		while(i[j].sizeX>ext.max.max_cube_map_texture_size)
		{
			IMAGE ret = resizeToHalfXY( i[j]);
			delete [] i[j].data;		
			if(ret.data==NULL)return 0;
			i[j] = ret;
		}
	}

	if(name!=NULL)delete[]name;
	name = new char[strlen(directory)+1];
	strcpy(name,directory);

	if(id!=0)glDeleteTextures( 1, &id);
	glGenTextures(1, &id);
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, id);

	if(ext.EXT_texture_edge_clamp)
	{	// vyuzijeme rozsirenie GL_SGI_texture_edge_clamp ( GL_EXT_texture_edge_clamp )
		// pri texturavani v blizkosti hrany sa farba hrany nepouziva, texturuje sa iba z textury
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere v (zvislom) neopakuju
	}
	else
	{
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP );		// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP );		// nastavuje ze textury sa v smere v (zvislom) neopakuju
	}

	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	switch(filter)
	{
	case 0:
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		break;
	case 1:
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		break;
	case 2:		// bilinear
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		break;
	case 3:		// trilinear
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		break;
	default:
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}
	
	unsigned int format;
	int internalformat;
	unsigned int position;
	for( j=0; j<6; j++)
	{
		switch(j)
		{
		case 0: position=GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB; break;
		case 1: position=GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB; break;
		case 2: position=GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB; break;
		case 3: position=GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB; break;
		case 4: position=GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB; break;
		case 5: position=GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB; break;
		}

		if(i[j].planes==1)
		{
			format = GL_ALPHA;
			if(compression && ext.ARB_texture_compression) internalformat = GL_COMPRESSED_ALPHA_ARB;
			else internalformat = GL_ALPHA8;
		}
		else if(i[j].planes==3)
		{
			if(i[j].isBGR&&ext.EXT_bgra) format = GL_BGR_EXT;
			else 
			{
				if(i[j].isBGR)BGRtoRGB(i[j]);
				format = GL_RGB;
			}
			if(compression && ext.ARB_texture_compression) internalformat = GL_COMPRESSED_RGB_ARB;
			else internalformat = GL_RGB8;
		}
		else if(i[j].planes==4)
		{
			if(i[j].isBGR&&ext.EXT_bgra) format = GL_BGRA_EXT;
			else 
			{
				if(i[j].isBGR)BGRtoRGB(i[j]);
				format = GL_RGBA;
			}
			if(compression && ext.ARB_texture_compression) internalformat = GL_COMPRESSED_RGBA_ARB;
			else internalformat = GL_RGBA8;
		}

		if( position==GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB || position==GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB || \
			position==GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB || position==GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB )
		{
			FlipMirror( i[j]);
		}
		if(filter<2)
		{
			glTexImage2D(position, 0, internalformat, i[j].sizeX, i[j].sizeY, 0, format, GL_UNSIGNED_BYTE, i[j].data);
		}
		else
		{			// mipmap
			if(ext.SGIS_generate_mipmap)
			{
				glHint(GL_GENERATE_MIPMAP_HINT_SGIS, GL_NICEST);
				glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_GENERATE_MIPMAP_SGIS, TRUE);
				glTexImage2D(position, 0, internalformat, i[j].sizeX, i[j].sizeY, 0, format, GL_UNSIGNED_BYTE, i[j].data);
			}
			else
				gluBuild2DMipmaps(position, internalformat, i[j].sizeX, i[j].sizeY, format, GL_UNSIGNED_BYTE, i[j].data);
		}
	}
	
	for( j=0; j<6; j++)if(i[j].data!=NULL)delete[] i[j].data;
	return 1;
}

void texture::FlipMirror(IMAGE &i)		// up - down, right - left
{
	if(i.planes<3 || i.data==NULL)return;
	unsigned char *data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(data==NULL)return;

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			data[(y*i.sizeX+x)*i.planes+0]=i.data[( (i.sizeY-y-1)*i.sizeX+i.sizeX-x-1)*i.planes+0];
			data[(y*i.sizeX+x)*i.planes+1]=i.data[( (i.sizeY-y-1)*i.sizeX+i.sizeX-x-1)*i.planes+1];
			data[(y*i.sizeX+x)*i.planes+2]=i.data[( (i.sizeY-y-1)*i.sizeX+i.sizeX-x-1)*i.planes+2];
		}
	}
	delete [] i.data;
	i.data = data;
}

IMAGE texture::LoadBMP8bit(FILE *f, IMAGE &i, BITMAPFILEHEADER &fileheader, BITMAPINFOHEADER &infoheader)
{
	RGBQUAD pal[256];

	if(infoheader.biBitCount!=8)return i;
	if(fread( pal, sizeof(RGBQUAD), infoheader.biClrUsed, f)!=infoheader.biClrUsed)
	{
		fclose(f);
		return i;
	}

	unsigned char* data;
	int line_size = infoheader.biWidth;
	if(line_size%4)line_size += 4 - line_size%4;
	data = new unsigned char[line_size*infoheader.biHeight];
	if(data==NULL){	fclose(f); return i; }
	if(!fread( data, line_size*infoheader.biHeight, 1, f))
	{	
		fclose(f);
		delete [] data;
		return i; 
	}
	fclose(f);

	i.sizeX=infoheader.biWidth;
	i.sizeY=infoheader.biHeight;
	i.planes=3;
	i.isBGR=0;

	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL){ delete [] data; return i; }

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(y*i.sizeX+x)*i.planes+0] = pal[data[y*line_size+x]].rgbRed;
			i.data[(y*i.sizeX+x)*i.planes+1] = pal[data[y*line_size+x]].rgbGreen;
			i.data[(y*i.sizeX+x)*i.planes+2] = pal[data[y*line_size+x]].rgbBlue;
		}
	}
	if(data!=NULL)delete []data;
	return i;
}

IMAGE texture::LoadBMP4bit(FILE *f, IMAGE &i, BITMAPFILEHEADER &fileheader, BITMAPINFOHEADER &infoheader)
{
	RGBQUAD pal[16];

	if(infoheader.biBitCount!=4)return i;
	if(fread( pal, sizeof(RGBQUAD), infoheader.biClrUsed, f)!=infoheader.biClrUsed)
	{
		fclose(f);
		return i;
	}

	unsigned char* data;
	int line_size = infoheader.biWidth/2+infoheader.biWidth%2;
	if(line_size%4)line_size += 4 - line_size%4;
	data = new unsigned char[line_size*infoheader.biHeight];
	if(data==NULL){	fclose(f); return i; }
	if(!fread( data, line_size*infoheader.biHeight, 1, f))
	{	
		fclose(f);
		delete [] data;
		return i; 
	}
	fclose(f);

	i.sizeX=infoheader.biWidth;
	i.sizeY=infoheader.biHeight;
	i.planes=3;
	i.isBGR=0;

	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL){ delete [] data; return i; }

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			int index;
			if(x%2) index = data[y*line_size+x/2]&0x0F;
			else index = (data[y*line_size+x/2]&0x0F0)>>4;
			i.data[(y*i.sizeX+x)*i.planes+0] = pal[index].rgbRed;
			i.data[(y*i.sizeX+x)*i.planes+1] = pal[index].rgbGreen;
			i.data[(y*i.sizeX+x)*i.planes+2] = pal[index].rgbBlue;
		}
	}
	if(data!=NULL)delete []data;

	return i;
}

IMAGE texture::LoadBMP1bit(FILE *f, IMAGE &i, BITMAPFILEHEADER &fileheader, BITMAPINFOHEADER &infoheader)
{
	RGBQUAD pal[2];

	if(infoheader.biBitCount!=1)return i;
	if(fread( pal, sizeof(RGBQUAD), infoheader.biClrUsed, f)!=infoheader.biClrUsed)
	{
		fclose(f);
		return i;
	}

	unsigned char* data;
	int line_size = infoheader.biWidth/8;
	if(infoheader.biWidth%8)line_size++;
	if(line_size%4)line_size += 4 - line_size%4;
	data = new unsigned char[line_size*infoheader.biHeight];
	if(data==NULL){	fclose(f); return i; }
	if(!fread( data, line_size*infoheader.biHeight, 1, f))
	{	
		fclose(f);
		delete [] data;
		return i; 
	}
	fclose(f);

	i.sizeX=infoheader.biWidth;
	i.sizeY=infoheader.biHeight;
	i.planes=3;
	i.isBGR=0;

	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL){ delete [] data; return i; }

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			unsigned char index=data[y*line_size+x/8];
			index >>= (7-x%8);
			index &= 0x01;
			i.data[(y*i.sizeX+x)*i.planes+0] = pal[index].rgbRed;
			i.data[(y*i.sizeX+x)*i.planes+1] = pal[index].rgbGreen;
			i.data[(y*i.sizeX+x)*i.planes+2] = pal[index].rgbBlue;
		}
	}
	if(data!=NULL)delete []data;

	return i;
}


IMAGE texture::resize(const IMAGE &in, int newx, int newy)
{
	IMAGE i;
	float *d;

	i.sizeX = newx;
	i.sizeY = newy;
	i.planes = in.planes;
	i.isBGR = in.isBGR;
	i.data = new unsigned char[i.sizeX*i.sizeY*i.planes];
	if(!i.data)return i;

	d = new float[in.sizeX*in.sizeY*in.planes];
	if(!d)
	{
		delete [] i.data;
		i.data = NULL;
		return i;
	}

	for(int j=0; j<in.sizeX*in.sizeY*in.planes; j++) d[j] = (float)in.data[j];

	float xf,yf;
	int ix,iy;		// index do vstupneho obrazka
	for(int y=0; y<newy; y++)
	{
		yf = y/(float)(newy-1);
		yf *= (float) in.sizeY;
		iy = (int) floor(yf);		// vrati mensiu celu cast 2.8 -> 2.0
		yf = (float)fmod(yf,1.0);			// zvysok po deleni 1
		for(int x=0; x<newx; x++)
		{
			xf = x/(float)(newx-1);
			xf *= (float) in.sizeX;
			ix = (int) floor(xf);		// vrati mensiu celu cast 2.8 -> 2.0
			xf = (float)fmod(xf,1.0);			// zvysok po deleni 1
		
			for(int c=0; c<in.planes; c++)
			{
				float a,b;
				if(ix<in.sizeX-1 && iy<in.sizeY-1)
				{
					a = (1.f-xf)*d[(iy*in.sizeX+ix)*in.planes+c]+xf*d[(iy*in.sizeX+ix+1)*in.planes+c];
					b = (1.f-xf)*d[((iy+1)*in.sizeX+ix)*in.planes+c]+xf*d[((iy+1)*in.sizeX+ix+1)*in.planes+c];
					a = (1.f-yf)*a+yf*b;
				}
				else
				{
					if(ix<in.sizeX-1)		// iy >= in.sizeY-1
					{
						iy = in.sizeY-1;
						a = (1.f-xf)*d[(iy*in.sizeX+ix)*in.planes+c]+xf*d[(iy*in.sizeX+ix+1)*in.planes+c];
					}
					else if(iy<in.sizeY-1)	// ix >= in.sizeX-1 && iy<in.sizeY-1
					{
						ix = in.sizeX-1;
						a = (1.f-yf)*d[(iy*in.sizeX+ix)*in.planes+c]+yf*d[((iy+1)*in.sizeX+ix)*in.planes+c];
					}
					else
					{
						ix = in.sizeX-1;
						iy = in.sizeY-1;
						a = d[(iy*in.sizeX+ix)*in.planes+c];
					}
				}
				i.data[(y*i.sizeX+x)*i.planes+c] = (unsigned char) a;
			}
		}
	}

	delete [] d;
	return i;
}

IMAGE texture::resizeToHalfX(const IMAGE &in)
{
	IMAGE i=in;
	i.data = NULL;

	if(in.sizeX<=1)return i;

	i.sizeX >>=1;		// /2
	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL)return i;

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(y*i.sizeX+x)*i.planes+0] = ((unsigned int)in.data[(y*in.sizeX+2*x)*in.planes+0]+(unsigned int)in.data[(y*in.sizeX+2*x+1)*in.planes+0])>>1;
			if(i.planes>1)i.data[(y*i.sizeX+x)*i.planes+1] = ((unsigned int)in.data[(y*in.sizeX+2*x)*in.planes+1]+(unsigned int)in.data[(y*in.sizeX+2*x+1)*in.planes+1])>>1;
			if(i.planes>2)i.data[(y*i.sizeX+x)*i.planes+2] = ((unsigned int)in.data[(y*in.sizeX+2*x)*in.planes+2]+(unsigned int)in.data[(y*in.sizeX+2*x+1)*in.planes+2])>>1;
			if(i.planes>3)i.data[(y*i.sizeX+x)*i.planes+3] = ((unsigned int)in.data[(y*in.sizeX+2*x)*in.planes+3]+(unsigned int)in.data[(y*in.sizeX+2*x+1)*in.planes+3])>>1;
		}
	}
	return i;
}

IMAGE texture::resizeToHalfY(const IMAGE &in)
{
	IMAGE i=in;
	i.data = NULL;
	if(in.sizeY<=1)return i;

	i.sizeY >>=1;		// /2
	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL)return i;

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(y*i.sizeX+x)*i.planes+0] = ((unsigned int)in.data[(2*y*in.sizeX+x)*in.planes+0]+(unsigned int)in.data[((2*y+1)*in.sizeX+x)*in.planes+0])>>1;
			if(i.planes>1)i.data[(y*i.sizeX+x)*i.planes+1] = ((unsigned int)in.data[(2*y*in.sizeX+x)*in.planes+1]+(unsigned int)in.data[((2*y+1)*in.sizeX+x)*in.planes+1])>>1;
			if(i.planes>2)i.data[(y*i.sizeX+x)*i.planes+2] = ((unsigned int)in.data[(2*y*in.sizeX+x)*in.planes+2]+(unsigned int)in.data[((2*y+1)*in.sizeX+x)*in.planes+2])>>1;
			if(i.planes>3)i.data[(y*i.sizeX+x)*i.planes+3] = ((unsigned int)in.data[(2*y*in.sizeX+x)*in.planes+3]+(unsigned int)in.data[((2*y+1)*in.sizeX+x)*in.planes+3])>>1;
		}
	}
	return i;
}

IMAGE texture::resizeToHalfXY(const IMAGE &in)
{
	IMAGE i=in;
	i.data = NULL;
	if(in.sizeX<=1 && in.sizeY<=1)return i;

	i.sizeX >>=1;		// /2
	i.sizeY >>=1;		// /2
	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL)return i;

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(y*i.sizeX+x)*i.planes+0] = (
							(unsigned int)in.data[(2*y*in.sizeX+2*x)*in.planes+0]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x)*in.planes+0]+
							(unsigned int)in.data[(2*y*in.sizeX+2*x+1)*in.planes+0]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x+1)*in.planes+0])>>2;
			if(i.planes>1)i.data[(y*i.sizeX+x)*i.planes+1] = (
							(unsigned int)in.data[(2*y*in.sizeX+2*x)*in.planes+1]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x)*in.planes+1]+
							(unsigned int)in.data[(2*y*in.sizeX+2*x+1)*in.planes+1]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x+1)*in.planes+1])>>2;
			if(i.planes>2)i.data[(y*i.sizeX+x)*i.planes+2] = (
							(unsigned int)in.data[(2*y*in.sizeX+2*x)*in.planes+2]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x)*in.planes+2]+
							(unsigned int)in.data[(2*y*in.sizeX+2*x+1)*in.planes+2]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x+1)*in.planes+2])>>2;
			if(i.planes>3)i.data[(y*i.sizeX+x)*i.planes+3] = (
							(unsigned int)in.data[(2*y*in.sizeX+2*x)*in.planes+3]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x)*in.planes+3]+
							(unsigned int)in.data[(2*y*in.sizeX+2*x+1)*in.planes+3]+
							(unsigned int)in.data[((2*y+1)*in.sizeX+2*x+1)*in.planes+3])>>2;
		}
	}
	return i;
}

IMAGE texture::shift(const IMAGE &in)		// jas * 0.5
{
	IMAGE i=in;

	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL)return i;

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(y*i.sizeX+x)*i.planes+0] = in.data[(y*i.sizeX+x)*i.planes+0]>>1;
			i.data[(y*i.sizeX+x)*i.planes+1] = in.data[(y*i.sizeX+x)*i.planes+1]>>1;
			i.data[(y*i.sizeX+x)*i.planes+2] = in.data[(y*i.sizeX+x)*i.planes+2]>>1;
		}
	}
	return i;
}

IMAGE texture::invShift(const IMAGE &in)	// invertovana a potom jas*0.5
{
	IMAGE i=in;

	i.data = new unsigned char[i.planes*i.sizeX*i.sizeY];
	if(i.data==NULL)return i;

	for(int y=0; y<i.sizeY; y++)
	{
		for(int x=0; x<i.sizeX; x++)
		{
			i.data[(y*i.sizeX+x)*i.planes+0] = (255-in.data[(y*i.sizeX+x)*i.planes+0])>>1;
			i.data[(y*i.sizeX+x)*i.planes+1] = (255-in.data[(y*i.sizeX+x)*i.planes+1])>>1;
			i.data[(y*i.sizeX+x)*i.planes+2] = (255-in.data[(y*i.sizeX+x)*i.planes+2])>>1;
		}
	}
	return i;
}

void texture::setForCopyToTexture(int x, int y, int repeat, int filter)
{
	if(id!=0)glDeleteTextures( 1, &id);
	glGenTextures(1, &id);
	glBindTexture(GL_TEXTURE_2D, id);

	if(repeat)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	}
	else
	{
		if(ext.EXT_texture_edge_clamp)
		{	// vyuzijeme rozsirenie GL_SGI_texture_edge_clamp ( GL_EXT_texture_edge_clamp )
			// pri texturavani v blizkosti hrany sa farba hrany nepouziva, texturuje sa iba z textury
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere v (zvislom) neopakuju
		}
		else
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );		// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );		// nastavuje ze textury sa v smere v (zvislom) neopakuju
		}
	}

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	switch(filter)
	{
	case 0:		// nearest
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		break;
	case 1:		// linear
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		break;
	case 2:		// bilinear
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		break;
	case 3:		// trilinear
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		break;
	default:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}

	if(filter<2)
	{
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, x, y, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
	}
	else
	{			// mipmap
		if(ext.SGIS_generate_mipmap)
		{
			glHint(GL_GENERATE_MIPMAP_HINT_SGIS, GL_NICEST);
			glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, TRUE);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, x, y, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
		}
		else
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB8, x, y, GL_RGB, GL_UNSIGNED_BYTE, NULL);
	}
}

void texture::setCubeForCopyToTexture(int x, int y, int repeat, int filter)
{
	// right, left, top, bottom, back, front
	if(id!=0)glDeleteTextures( 1, &id);
	glGenTextures(1, &id);
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, id);

	if(ext.EXT_texture_edge_clamp)
	{	// vyuzijeme rozsirenie GL_SGI_texture_edge_clamp ( GL_EXT_texture_edge_clamp )
		// pri texturavani v blizkosti hrany sa farba hrany nepouziva, texturuje sa iba z textury
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );	// nastavuje ze textury sa v smere v (zvislom) neopakuju
	}
	else
	{
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP );		// nastavuje ze textury sa v smere u (vodorovnom) neopakuju
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP );		// nastavuje ze textury sa v smere v (zvislom) neopakuju
	}

	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	switch(filter)
	{
	case 0:
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		break;
	case 1:
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		break;
	case 2:		// bilinear
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		break;
	case 3:		// trilinear
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		break;
	default:
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}
	
	unsigned int position;
	for(int j=0; j<6; j++)
	{
		switch(j)
		{
		case 0: position=GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB; break;
		case 1: position=GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB; break;
		case 2: position=GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB; break;
		case 3: position=GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB; break;
		case 4: position=GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB; break;
		case 5: position=GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB; break;
		}

	//	if( position==GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB || position==GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB || \
	//		position==GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB || position==GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB )
	//	{
		//	FlipMirror( i[j]);
	//	}
		if(filter<2)
		{
			glTexImage2D(position, 0, GL_RGB8, x, y, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
		}
		else
		{			// mipmap
			if(ext.SGIS_generate_mipmap)
			{
				glHint(GL_GENERATE_MIPMAP_HINT_SGIS, GL_NICEST);
				glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_GENERATE_MIPMAP_SGIS, TRUE);
				glTexImage2D(position, 0, GL_RGB8, x, y, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
			}
			else
				gluBuild2DMipmaps(position, GL_RGB8, x, y, GL_RGB, GL_UNSIGNED_BYTE, NULL);
		}
	}
}
